import IKGlobals from './../IKGlobals';
//
import AWS from "aws-sdk";
import Utils from "./IKUtils";
//
export function IKMissingParamResponse(paramName) {
  console.warn(`Invalid request - Path parameter ${paramName} is missing.`);
  return (new IKResponse(400, {err: `Invalid request. Path parameter ${paramName} is missing.`, errCode: IKGlobals.ErrorCode_MissingParam}));
}
export function IKBadRequestResponse(msg, errCode) {
  console.warn(`Bad request - ${msg}`);
  return (new IKResponse(400, {err: msg, ...(errCode ? {errCode: errCode} : {})}));
}
export function IKUnauthorizedResponse(msg, errCode) {
  console.warn(`Denying request - ${msg}`);
  return (new IKResponse(401, {err: msg, ...(errCode ? {errCode: errCode} : {})}));
}
export function IKSuccessResponse(msg) { return (new IKResponse(200, (msg ? msg : {}))); }
//
export default class IKResponse {
  constructor(statusCode, body) {
    this._statusCode = statusCode;
    this._body = body;
    this._streamingOut = false;
  }
  getCode() { return this._statusCode; }
  getBody() { return this._body; }
  appendIntoBody(key, value) { this._body[key] = value; }
  build(context, requestID, onQueue) {
    //Check if not streaming
    if (this._streamingOut) return;
    //append default fields
    if (requestID) this.appendIntoBody('transactionID', requestID); //append transaction ID
    //build response
    let b = {
      statusCode: this._statusCode,
      body: JSON.stringify(this._body),
      headers: {
        "Access-Control-Allow-Origin": "*",
        "Access-Control-Allow-Credentials": true,
        "Content-Type": "application/json"
      }
    };
    //log response and respond to context
    console.debug(b)
    if (!onQueue) context.succeed(b);
  }
  pipe(context, requestID) {
    //Check if not streaming
    if (this._streamingOut) return;
    this._streamingOut = true;
    //append default fields
    if (requestID) this.appendIntoBody('transactionID', requestID); //append transaction ID
    //build response
    let b = {
      statusCode: this._statusCode,
      body: this._body,
      headers: {
        "Access-Control-Allow-Origin": "*",
        "Access-Control-Allow-Credentials": true,
        "Content-Type": "application/json"
      }
    };
    //log response and respond to context
    console.debug('Streaming response', b);
    context.succeed(b);
  }
}
