import AWS from "aws-sdk";
import HTTPS from "https";
import Utils from "./../API/IKUtils";
//
import IKGlobals from './../IKGlobals';
//
import IKDBQueryPut from './IKDBQueryPut';
import IKDBQueryUpdate from './IKDBQueryUpdate';

export default class IKDB {
  constructor(config) {
    this.tableName = config.tableName;
    this.region = config.region;
    if (config && this.tableName) {
      console.debug(`Using table: ${this.tableName} on region: ${this.region}`);
      this._configure();
    }
  }

  _configure() {
    //setup db
    if (Utils.isOffline()) {
      console.debug("Starting offline database connection");
      this.connection = new AWS.DynamoDB(IKGlobals.DynamoDBLocalConfig);
    } else {
      console.debug("Starting remote database connection");
      //setup ssl
      const sslAgent = new HTTPS.Agent({ keepAlive: true, maxSockets: 50, rejectUnauthorized: true });
      sslAgent.setMaxListeners(50);
      //initialize connection
      AWS.config.update({httpOptions: sslAgent, region: this.region});
      this.connection = new AWS.DynamoDB();
    }
  }

  //Transaction support
  runQueuedQueries() {

  }

  //main interface
  async runQuery(qry) { return await qry.run(this); }
  async rawRunQuery(qry) { return await qry.rawRun(this); }
  async queueQuery(qry) {
    if (qry instanceof IKDBQueryPut || qry instanceof IKDBQueryUpdate) {

    } else { console.error("Unknown query type! - preventing from queueing it!"); }
  }
}
