//Components
import IKTransaction from './IKTransaction';
import IKResponse, { IKBadRequestResponse } from './API/IKResponse';
//
const pathToRegexp = require('path-to-regexp');
//
export default class IKRouter {
  constructor(routes, config) {
    this.routes = routes;
    this.config = config;
  }

  //Main handler
  async handleEvent(event, context) {
    //init transaction
    await (new IKTransaction(event, context, this.config)).execute(async transaction => {
      //get path and tries to match an defined route with existing request
      const path = transaction.request.getPath();
      for (let route of this.routes) {
        //Check for possible paths
        const resp = this._routeMatches(route.route, route.method, transaction, route.handler);
        if (resp) {
          console.debug("Router accepted route:", route)
          return await resp(transaction);
        }
      }
      //No route found :/
      return new IKResponse(404, {err: 'Route not found!'});
    });
  }
  //Helper function to match route and return handler if matches
  _routeMatches(exp, method, transaction, handler) {
    const path = transaction.request.getPath();
    let keys = [];
    const result = pathToRegexp(exp, keys).exec(path);
    if (result) {
      const reqMethod = transaction.request.getMethod();
      if (reqMethod.toLowerCase() == method.toLowerCase()) {
        //Fix path params on proxy
        transaction.request.requestEvent.pathParameters = {};
        keys.forEach((key, index) => transaction.request.requestEvent.pathParameters[key.name] = result[index + 1]);
        //
        return handler;
      }
    }
  }
}
