//Components
import Utils  from "./API/IKUtils";
import IKDB from "./Database/IKDB";
import IKLogger from "./Logger/IKLogger";
import IKGlobals from './IKGlobals';
import IKValidation from './Validation/IKValidation';
import IKRequest from './API/IKRequest';
import IKResponse, { IKBadRequestResponse } from './API/IKResponse';
//
export default class IKTransaction {
  constructor(event, context, config, _isQueue) {
    //
    this._event = event;
    this._context = context;
    //queue support
    this._onQueue = _isQueue;
    this._queueResp = null;
    //
    this.logger = new IKLogger({/*COFIG S3/SQS HERE*/}, Utils.logLevel(), (context.awsRequestId ? context.awsRequestId : (event.requestContext ? event.requestContext.requestId : 'unknown')));
    this.request = new IKRequest(this._event, this._context);
    this.db = new IKDB(config.database);
    this.validator = new IKValidation(config.validation);
  }

  //Main interface
  async execute(executionFunc) {
    await this._executeLoggerFlush(async ()=>{
      await this._executeDBTransactions(async ()=>{
        return await this._execute(executionFunc);
      });
    });
    if (this._onQueue) return this._queueResp;
  }
  //Executions
  async _execute(executionFunc) {
    let executionFailed = true; //failled til we say no!
    //safe execution handler
    try { //Execute
      console.debug("Starting main request code");
      this._queueResp = await executionFunc(this);
      //Answer client
      if (this._queueResp && this._queueResp instanceof IKResponse) {
        this._queueResp.build(this._context, this.request.getRequestID(), this._onQueue);
      } else {
        this._queueResp = IKBadRequestResponse(IKGlobals.ErrorResponseInvalidServerResponse, IKGlobals.ErrorCode_APIError)
        this._queueResp.build(this._context, this.request.getRequestID(), this._onQueue);
        console.error("Invalid response object from main request code.");
      }
      executionFailed = false;
    } catch (e) { /*EXECUTION FAIL*/
      console.error('Exception when executing main request code.');
      console.exception(e);
      if (executionFailed) {
        this._queueResp = IKBadRequestResponse(IKGlobals.ErrorResponseUnhandledError, IKGlobals.ErrorCode_APIError);
        this._queueResp.build(this._context, this.request.getRequestID(), this._onQueue);
      }
    } return executionFailed;
  }
  async _executeDBTransactions(safeExecution) {
    try {
      let executionFailed = await safeExecution();
      if (!executionFailed) await this.db.runQueuedQueries();
      else console.log("Skipping pending DB transactions. Main code failed!");
    } catch (e) {
      console.error('Exception when executing DB transactions.');
      console.log(e);
    }
  }
  async _executeLoggerFlush(safeExecution) {
    try {
      await safeExecution();
      await this.logger.flushLogs();
    } catch (e) {
      console.error('Exception when flushing logs.');
      console.exception(e);
    } finally {
      console.debug("Transaction ended");
    }
  }
  /* stream support */
  async pipeOut(streamResponse) {
    if (!this._isQueue) streamResponse.pipe(this._context, this.request.getRequestID());
    else console.error('Can\'t pipeout response when in queue!');
  }
}
